/**
 * Blink.cpp 
 */
// incluímos las librerías de arduino
#include "Arduino.h"

// incluimos nuestra librería
#include "Blink.h"


// Constructores //////////////////////////////////////////////////////
/**
 *  Función que manipula la creación y el ajuste de las instancias 
 *  cuando no se pasan parámetros, se toman valores por defecto
 */
Blink::Blink()
{  
  this->_led = pinLed;
  this->_retardo = retardoLed;   // retardo en milis

  // inicializa el led como salida
  pinMode(this->_led, OUTPUT);

  // ponemos el led en un estado conocido
  digitalWrite(this->_led, AP_LED);   // led apagado    
}

/**
 *  Función que manipula la creación y el ajuste de las instancias 
 *  cuando solamente se pasa la patilla a utilizar
 */
Blink::Blink(int led)
{  
  this->_led = led;
  this->_retardo = retardoLed;   // retardo en milis

  // inicializa el led como salida
  pinMode(this->_led, OUTPUT);

  // ponemos el led en un estado conocido
  digitalWrite(this->_led, AP_LED);   // led apagado    
}

/**
 *  Función que manipula la creación y el ajuste de las instancias 
 *  cuando se pasan todos los valores.
 *  @param pin entero que indica la patilla a utilizar
 *  @param 
  */
Blink::Blink(int led, unsigned long int retardo)
{  
  this->_led = led;
  this->_retardo = retardo;   // retardo en milis

  // inicializa el led como salida
  pinMode(_led, OUTPUT);

  // ponemos el led en un estado conocido
  digitalWrite(_led, AP_LED);   // led apagado    
}



// Métodos Públicos ////////////////////////////////////////////////////// 
/**
 * blink
 * 
 * Hace parpadear el led a intervalos regulares
 */
void Blink::blink(void) 
{
  static bool encendido = AP_LED;
  static unsigned long tiempo = millis();
  
  if (millis() - tiempo >= this->_retardo) {
    
    tiempo = millis();
    
    if (encendido) {
      encendido = false;
      digitalWrite(this->_led, AP_LED);    // apaga el led
    }
    else {
      encendido = true;
      digitalWrite(this->_led, EN_LED);    // apaga el led
    }    
  }
}



